// Provide wrapper APIs for calling the scintilla
// Copyright (c) 2013 Mentor Graphics Corporation
// The license.terms file describes the conditions under which this software may be distributed.

#ifndef SCIWRAPPERS_H
#define SCIWRAPPERS_H

#include "Scintilla.h"
#include "ScintillaTk.h"

#ifdef __cplusplus
extern "C" {
#endif

// Short-hand command API
sptr_t sci_cmd(ScintillaObject *sci, int msg, uptr_t wparam = 0, sptr_t lparam = 0);
   

// ACTIONs
void sci_add_text           (ScintillaObject *sci, sptr_t pos, const char *s);
void sci_delete_text        (ScintillaObject *sci, sptr_t pos1, sptr_t pos2);
void sci_enable_margin_click(ScintillaObject *sci, int m, bool enable);
void sci_goto_pos           (ScintillaObject *sci, sptr_t pos);
bool sci_has_selection      (ScintillaObject *sci);
void sci_redo               (ScintillaObject *sci);
void sci_reset_undo_history (ScintillaObject *sci);
void sci_select             (ScintillaObject *sci, sptr_t pos1, sptr_t pos2, bool additive=false);
void sci_show_ws            (ScintillaObject *sci, bool show);
void sci_toggle_fold        (ScintillaObject *sci, int line);
void sci_undo               (ScintillaObject *sci);
void sci_zoom_in            (ScintillaObject *sci);
void sci_zoom_out           (ScintillaObject *sci);

// GETs
int    sci_get_column         (ScintillaObject *sci, sptr_t pos);
sptr_t sci_get_current_pos    (ScintillaObject *sci);
int    sci_get_foldlevel      (ScintillaObject *sci, int line);
int    sci_get_is_modified    (ScintillaObject *sci);
sptr_t sci_get_length         (ScintillaObject *sci);
sptr_t sci_get_line           (ScintillaObject *sci, sptr_t pos);
sptr_t sci_get_line_count     (ScintillaObject *sci);
int    sci_get_line_length    (ScintillaObject *sci, int line);
int    sci_get_line_markers   (ScintillaObject *sci, int line);
sptr_t sci_get_lineend_pos    (ScintillaObject *sci, int line);
int    sci_get_margin_mask    (ScintillaObject *sci, int m);
int    sci_get_margin_width   (ScintillaObject *sci, int m);
sptr_t sci_get_pos            (ScintillaObject *sci, int line);
sptr_t sci_get_pos_from_xy    (ScintillaObject *sci, int x, int y, bool close = false);
int    sci_get_text_height    (ScintillaObject *sci, int line);
int    sci_get_text_width     (ScintillaObject *sci, const char *str);
int    sci_get_x_from_pos     (ScintillaObject *sci, sptr_t pos);
int    sci_get_y_from_pos     (ScintillaObject *sci, sptr_t pos);
int    sci_get_zoom           (ScintillaObject *sci);

// SETs
void sci_set_focus          (ScintillaObject *sci, bool turn_on);
void sci_set_fold_flags     (ScintillaObject *sci, int flags);
void sci_set_font           (ScintillaObject *sci, const char *family, int ptsize, bool bold, bool italic);
void sci_set_margin_mask    (ScintillaObject *sci, int m, int mask);
void sci_set_margin_width   (ScintillaObject *sci, int m, int width);
void sci_set_margin_type    (ScintillaObject *sci, int m, int type);
void sci_set_padding        (ScintillaObject *sci, int left, int right = -1);
void sci_set_readonly       (ScintillaObject *sci, bool ro);
void sci_set_style_bg       (ScintillaObject *sci, int id, XColor *c);
void sci_set_style_fg       (ScintillaObject *sci, int id, XColor *c);
void sci_set_tab_width      (ScintillaObject *sci, int width);

#ifdef __cplusplus
}
#endif

#endif
